package game;

import java.util.ArrayList;
import main.Art;
import main.Art.Bitmap;
import main.GameComponent;
import abstracts.Renderable;

public class HeadsUpDisplay extends Renderable {
	
	private SurvivalGame game;
	private ArrayList<PixelData> screen;
	private int cooldownTime;
	
	public HeadsUpDisplay(SurvivalGame game) {
		this.game = game;
		initialize();
		
	}
	
	public void initialize() {
		if (this.screen == null)
			this.screen = new ArrayList<PixelData>();
		else
			this.screen.clear();
		
		GameComponent gameComponent = this.game.getGameComponent();
		int[] data = gameComponent.displayPixels;
		for (int i = 0; i < data.length; i++) {
			int x = i % GameComponent.WIDTH;
			int y = i / GameComponent.HEIGHT;
			PixelData pixel = new PixelData();
			pixel.color = 0;
			pixel.x = x;
			pixel.y = y;
			this.screen.add(pixel);
		}
		
		this.setActive();
		gameComponent.setDisplayActive();
	}
	
	@Override
	public void tick() {
		if (this.isActive()) {
			cooldownTime = game.getPlayerCooldownTimer();
		}
	}
	
	@Override
	public void setInactive() {
		super.setInactive();
		this.game.getGameComponent().setDisplayInactive();
	}
	
	@Override
	public void render() {
		if (this.isActive()) {
			clear();
			render_cooldown(Art.cooldown, 3, 3);
			renderBack();
		}
	}
	
	private void clear() {
		for (int i = 0; i < this.screen.size(); i++) {
			this.screen.get(i).color = 0;
		}
	}
	
	private void renderBack() {
		int[] data = this.game.getGameComponent().displayPixels;
		for (int i = 0; i < this.screen.size(); i++) {
			data[i] = this.screen.get(i).color;
		}
	}
	
	private void render_cooldown(Bitmap bitmap, int xOffset, int yOffset) {
		for (int j = 0; j < bitmap.height; j++) {
			for (int i = 0; i < bitmap.width; i++) {
				screen.get((j + yOffset) * (GameComponent.WIDTH) + (i + xOffset)).color = bitmap.pixels[j * bitmap.width + i];
			}
		}
		xOffset += Art.cooldown.width + 2;
		for (int y = 0; y < Art.cooldown.height; y++) {
			for (int x = cooldownTime; x > 0; x--) {
				screen.get((y + yOffset) * GameComponent.WIDTH + (x + xOffset)).color = 0x66D97827;
			}
		}
	}
}
